@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: braces.c,v 1.32 1999/02/17 17:28:45 will Exp $";
+static const char rcsid[] = "$Id: braces.c,v 1.33 2004/04/30 17:00:58 will Exp $";
 /*
  * brace.c: csh brace expansion -- includes pieces of
  * [t]csh's sh.misc.c and sh.glob.c.
@@ -362,7 +362,7 @@ blkprint(av)
 char **av;
 {
     int i;
-    fprintf(stderr, "blkprint(%#lx):\n", av);
+    fprintf(stderr, "blkprint(%#lx):\n", (unsigned long) av);
     if (!av)
 	return;
     for (i = 0;  *av; av++, i++) {
