@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: colon.c,v 1.29 2004/04/30 17:00:58 will Exp $";
+static const char rcsid[] = "$Id: colon.c,v 1.31 2007/03/07 15:00:18 will Exp $";
 
 #include "super.h"
 #include "version.h"
@@ -183,6 +183,9 @@ char *word;	/* opt=xxx or condition~Pattern or PermittedUserPattern */
 	else
 	    pattern = s+1;	/* s points to "~"; advance past to pattern */
 
+	if (balancedbraces(pattern) != 0) {
+	    return Error(0, 0, "%tUnbalanced braces in `%s'.\n", pattern);
+	}
 	if ((i=globbraces(pattern, 1, &glob)) != 0) {
 	    /* Global condition */
 	    return Error(0, 0, "%tMissing `%c'.\n", i);
