@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: error.c,v 1.78 2004/05/18 02:48:47 will Exp $";
+static const char rcsid[] = "$Id: error.c,v 1.79 2007/03/07 00:23:27 will Exp $";
 
 /*
  *	Copyright (c) 1993 by California Institute of Technology.
@@ -54,6 +54,7 @@ static const char rcsid[] = "$Id: error.c,v 1.78 2004/05/18 02:48:47 will Exp $"
  *	error_srcfile, error_line, error_nl: error_srcfile is a file in
  *			which an error was found, starting at line error_line
  *			and continuing over error_nl lines.
+ *	error_counter: incremented with every call to Error().
  * In detail:
 
  * If error_prog != NULL, then the message to stderr is preceded
@@ -133,6 +134,8 @@ char *error_log_prog = NULL;
 char *error_command = NULL;
 char *error_user = NULL;
 
+int error_counter = 0;
+
 int error_stderr = 1;
 
 int error_syslog = 0;
@@ -230,6 +233,8 @@ Error(
 
     error = errno;
 
+    error_counter++;
+
     /* Figure out line tagging */
     pctt = strncmp(fmt, "%t", 2) == 0;
     tag = ( pctt || error_tag ) ? taglines(3) : NULL;
