@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: options.c,v 1.129 2004/10/22 00:22:19 will Exp $";
+static const char rcsid[] = "$Id: options.c,v 1.131 2007/03/07 15:00:20 will Exp $";
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Process a global xxx=yyy option */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -192,6 +192,9 @@ options cannot be negated: <%s>\n", word);
 	    char **globlist;
 	    int i;
 
+	    if (balancedbraces(value) != 0) {
+		return Error(0, 0, "%tUnbalanced braces in `%s'.\n", value);
+	    }
 	    if ((i=globbraces(value, 1, &globlist)) != 0) {
 		/* Option argument */
 		DoDebug();
