@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: s_regex.c,v 1.9 1998/03/02 06:53:59 will Exp $";
+static const char rcsid[] = "$Id: s_regex.c,v 1.10 2004/04/30 17:00:58 will Exp $";
 /*
  * regex - Regular expression pattern matching
  *         and replacement
@@ -240,11 +240,12 @@ char *pat;
 	register int n;
 	int c1, c2;
 		
-	if (!pat || !*pat)
+	if (!pat || !*pat) {
 		if (sta)
 			return(0);
 		else
 			badpat("No previous regular expression");
+	}		
 	sta = NOP;
 
 	for (p = pat; *p; p++) {
