@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: super.c,v 1.596 2007/03/07 16:58:13 will Exp $";
+static const char rcsid[] = "$Id: super.c,v 1.613 2007/12/20 19:14:30 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -25,15 +25,17 @@ static const char rcsid[] = "$Id: super.c,v 1.596 2007/03/07 16:58:13 will Exp $
  *     $0 [-H | -f] [-F superfile] [ -T hh[:mm][/day] ] [-U uid] [-G gid]
  *		...lists allowed commands.
  *
- *     $0 [-o file] command [args...]
+ *     $0 [-r path] [-o path] [args...]
  *		...execute command.
  *
  *     $0 -b
  *		...lists builtin variables (useful when creating super.tab).
  *
- *     $0 [-d | -D] [-F superfile] [-T time] [-U uid] [-G gid] [-M mach] \
- *			[-o file] [--] [commandname [args...]]
- *		...debug mode.
+ *     $0 [-d | -D | -t] [-F superfile] [-T time] [-U uid] [-G gid] [-M mach] \
+ *			[-r reqpath] [--] [commandname [args...]]
+ *     $0 [-d | -D | -t] [-F superfile] [-T time] [-U uid] [-G gid] [-M mach] \
+ *			[-r reqpath] -o path [--] [args...]
+ *		...debug mode (-d, -D) or test mode (-t).
  *
  *     $0 -c [superfile]
  *		...just check syntax of the superfile.
@@ -44,6 +46,8 @@ static const char rcsid[] = "$Id: super.c,v 1.596 2007/03/07 16:58:13 will Exp $
  *	-d -- debug mode: show what would be done,
  *		but don't actually execute a command.
  *	-D -- verbose debug mode: same as -d, plus tell more about variables.
+ *	-t -- test mode: test if requested command is valid, but don't
+ *		execute it.  Exit code=0 if valid, else 1.
  *	-F superfile -- names the super file to use; for testing only.
  *		No command will actually be executed.
  *	-G gid -- act as if the invoking user was group gid; for testing only.
@@ -65,6 +69,19 @@ static const char rcsid[] = "$Id: super.c,v 1.596 2007/03/07 16:58:13 will Exp $
  *	-U uid -- act as if the invoking user was user uid; for testing only.
  *		No command will actually be executed.
  *	-V -- print version information, then exit.
+ *	-o file -- Per-user .supertab linking: if /path/to/xyz is a symlink
+ *		to a user's personal .supertab file, and the .supertab file
+ *		begins with #! /path/to/super -o, then the shell will invoke
+ *		super with arguments like super -o /path/to/xyz [args],
+ *		and super will treat this as 'super <username>:xyz [args].
+ *		MUST be last argument; everything following -o file is treated
+ *		as user's arguments.
+ *	-r path -- error if the FullPath isn't the same file as this path.
+ *		For sanity checking; a program can use
+ *			test "X$SUPERCMD" = "X$prog" || \
+ *			    exec /usr/local/bin/super -r $0 $prog ${1+"$@"}
+ *		and it will be an error if 'super <prog>' isn't going to
+ *		invoke this file, ie $0.
  * 
  * The super.tab file names each command that super will execute, and
  * says who can use it.  See super.tab.summary for an overview.
@@ -180,6 +197,7 @@ char *prog;				/* this program */
 int debug=0;				/* Set by the debug options flag */
 int use_stdin=0;			/* Set by the -S flag */
 int it_came_from_cmdline=0;		/* Set by -F/-T/-U/-G/-M flags */
+int test_mode=0;			/* Set by -t flag */
 int check_syntax=0;			/* Set by the -c options flag */
 int using_user_supertab = 0;		/* !0 means using a user's .supertab */
 SimpleList Var_Value = {NULL, NULL};
@@ -216,6 +234,7 @@ char **argv;
     extern char *error_prog;
     int iarg, givehelp, giveversion, verbosity, printvars;
     char *o_file;	/* The argument to -o xxx */
+    char *r_path;	/* The argument to -r path */
 
 #ifdef RLIMIT_CORE
     getrlimit(RLIMIT_CORE, &old_core_limit);
@@ -234,6 +253,7 @@ char **argv;
     debug = check_syntax = giveversion = givehelp = printvars = 0;
     verbosity = HELP_BASIC;	/* only matters if *givehelp != 0 */
     o_file = NULL;
+    r_path = NULL;
 
     error_srcfile = superfile;	/* error messages w/ line nums refer to this */
     error_line = -1;
@@ -270,11 +290,11 @@ so it can't check passwords on modern Unix systems.\n\n");
 	/* Invoked as super [options] cmd [arg ...]. */
 
 	iarg = do_options(argc, argv, &givehelp, &giveversion,
-			&verbosity, &printvars, &o_file);
+			&verbosity, &printvars, &o_file, &r_path);
 	if (iarg < 0 || (argv[0] == NULL && !(givehelp || giveversion))) {
 	    /* User screwed up; give minimal help */
 	    fprintf(stderr, "Type   %s -h   for usage information.\n", prog);
-	    exit(0);
+	    exit(1);
 	}
 	if (o_file) {
 	    /* Cmd is last component of the o_file path, e.g.
@@ -416,6 +436,8 @@ so it can't check passwords on modern Unix systems.\n\n");
 	    Error(0, 0,
 	   "Not applying nice increment = %d because of flag -F/-T/-U/-G/-M.\n",
 		rcl_nice_incr());
+	} else if (test_mode) {
+	    /* silently don't apply nice increment */
 	} else if (set_nice_incr() == -1) {
 	    return 1;
 	}
@@ -426,8 +448,11 @@ so it can't check passwords on modern Unix systems.\n\n");
      * to read a shadow password file or switch to another uid for writing
      * the timestamp file.
      */
-    if (check_auth(cmd) != 0)
-	return 1;
+    if (!test_mode) {
+	if (check_auth(cmd) != 0) {
+	    return 1;
+	}
+    }
 
     /* Set uid/gid if necessary */
     if (using_user_supertab) {
@@ -442,6 +467,13 @@ so it can't check passwords on modern Unix systems.\n\n");
     if (!envp)
 	return 1;
 
+    if (r_path) {
+	/* User supplied '-r path' check if it matches actual fullpath */
+	if (check_rpath(r_path, path) != 0) {
+	    return 1;
+	}
+    }
+
     /* Change directory if requested */
     if (status != -1)
 	status = set_chdir();
@@ -449,27 +481,34 @@ so it can't check passwords on modern Unix systems.\n\n");
     /* Set the umask value */
     set_umask();
 
-    if (debug) {
+    if (debug || it_came_from_cmdline) {
 	debug_print(path, arglist, envp, n_builtin);
-	if (status == -1) {
+    }
+
+    if (status == -1) {
+	if (debug || it_came_from_cmdline) {
 	    fprintf(stderr, "\n\t(This command would not be executed \
 due to previously-reported problem)\n");
-	} else {
-	    fprintf(stderr,
-	    "\n\t(Your command is ok, but isn't executed in debug mode.)\n");
 	}
-	return 0;
-    } else if (it_came_from_cmdline) {
-	debug_print(path, arglist, envp, n_builtin);
-	fprintf(stderr,
-	"\n\t(Your command is ok, but isn't executed because of one or more \
--F/-T/-U/-G/-M flags.)\n");
-	return 0;
-    }
-    if (status == -1) {
 	return 1;
     }
 
+    if (debug) {
+	fprintf(stderr,
+	    "\n\t(Your command is ok, but isn't executed in debug mode.)\n");
+	return 0;
+
+    } else if (it_came_from_cmdline) {
+	fprintf(stderr,
+	    "\n\t(Your command is ok, but isn't executed because of one or \
+more -F/-T/-U/-G/-M flags.)\n");
+	return 0;
+
+    } else if (test_mode) {
+	/* Command is ok, but isn't executed because we're in test mode */
+	return 0;
+    }
+
     /* Do checks required by a checkvar=a,b,c... list */
     if (check_var_value() != 0)
 	return 1;
@@ -560,11 +599,12 @@ due to previously-reported problem)\n");
  */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
-do_options(argc, argv, givehelp, giveversion, verbosity, printvars, o_file)
+do_options(argc, argv, givehelp, giveversion, verbosity, printvars,
+			o_file, r_path)
 int argc;
 char **argv;
 int *givehelp, *giveversion, *verbosity, *printvars;
-char **o_file;
+char **o_file, **r_path;
 {
     int iarg;
     char *mhost, *s, *t;
@@ -717,6 +757,21 @@ char **o_file;
 		    			pw->pw_uid, pw->pw_name, pw->pw_gid);
 		}
 		break;
+	    case 'r':
+		if (*(s+1)) {
+		    /* User gave `-rpath' */
+		    *r_path = s+1;
+		    s += strlen(s) - 1;
+		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
+		    /* User gave `-r path' */
+		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
+		    *r_path = argv[iarg];
+		} else {
+		    return Error(0, 0, "No file specified after option -r\n");
+		}
+		break;
 	    case 'o':
 		if (*(s+1)) {
 		    /* User gave `-ofile' */
@@ -739,6 +794,9 @@ char **o_file;
 		iarg++;
 		return iarg;
 	    case 'b': *printvars = 1; break;
+	    case 't':
+		test_mode = 1;
+		break;
 	    case 'd':
 		debug = 1;
 		break;
@@ -1691,6 +1749,35 @@ rcl_umask()
     return (localinfo.mask >= 0) ? localinfo.mask : globalinfo.mask;
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Check if -r path matches actual path.
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+int
+check_rpath(r_path, xpath)
+char *r_path;
+char *xpath;
+{
+    struct stat r_stat, xstat;
+    if (stat(r_path, &r_stat) == -1) {
+	return Error(1, 0, "Caller used `-r %-.500s', but stat of that \
+path failed: ", r_path);
+    }
+    if (stat(xpath, &xstat) == -1) {
+	return Error(1, 0, "Caller used `-r %-.500s'; the super.tab file \
+gave path `%s', and the stat of the latter file failed: ",
+	    	r_path, xpath);
+    }
+    if (r_stat.st_dev != xstat.st_dev || r_stat.st_ino != xstat.st_ino) {
+	return Error(0, 0, "File sanity-check mismatch: \
+caller used `-r %-.500s' (dev=%ld,inode=%ld); the super.tab file \
+gave path `%s' (dev=%ld,inode=%ld): these are not the same file.\n",
+	    r_path, (long) r_stat.st_dev, (long) r_stat.st_ino,
+	    xpath, (long) xstat.st_dev, (long) xstat.st_ino);
+    }
+
+    return 0;
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Change directory, if required */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
