@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: utils.c,v 1.155 2005/11/15 19:49:16 will Exp $";
+static const char rcsid[] = "$Id: utils.c,v 1.157 2007/03/07 14:48:15 will Exp $";
 #include "super.h"
 
 /* Variables table flag -- !0 means has been created */
@@ -1325,7 +1325,7 @@ char *vardefn;
     int lb = strlen(vardefn);
 
     if (!created_variables_table) {
-	if (!s_hcreate(1000))
+	if (!s_hcreate(HS_VARS, 1000))
 	    return Error(0, 0,
 		"%tCouldn't allocate hash table for variable processing\n");
 	created_variables_table = 1;
@@ -1352,12 +1352,12 @@ char *vardefn;
 	fprintf(stderr, "Adding variable `%s' = `%s'\n", item.key, item.data);
 
     /* Delete the existing value, if any */
-    if ((found_item = s_hsearch(item, FIND))) {
+    if ((found_item = s_hsearch(HS_VARS, item, FIND))) {
 	/* Assume all data allocated via malloc() */
 	free(found_item->data);
 	found_item->data = NULL;
     }
-    if (!s_hsearch(item, ENTER))
+    if (!s_hsearch(HS_VARS, item, ENTER))
 	return Error(0, 0,
 	"%tFailed to allocate space for hash-table entry for variable `%s'\n",
 		varname);
@@ -1618,7 +1618,7 @@ char *string;
 	if (varname) {
 	    /* Have a variable requiring expansion */
 	    wanted_item.key = varname;
-	    found_item = s_hsearch(wanted_item, FIND);
+	    found_item = s_hsearch(HS_VARS, wanted_item, FIND);
 	    if (!found_item) {
 		Error(0, 0, "%t\n\tNo such variable as `$%s'\n",
 			wanted_item.key);
@@ -1655,7 +1655,7 @@ char *varname;
 
     /* Have a variable requiring expansion */
     wanted_item.key = varname;
-    found_item = s_hsearch(wanted_item, FIND);
+    found_item = s_hsearch(HS_VARS, wanted_item, FIND);
     if (!found_item) {
 	return NULL;
     }
@@ -1871,6 +1871,23 @@ int len;
     return 0;
 }
 
+/*
+ * Check for balanced braces.
+ * Return 0 if balanced, else (nleft-nright).
+ */
+int balancedbraces(s)
+char *s;
+{
+    int n;
+    for (n=0; *s; s++) {
+	switch (*s) {
+	case '{': n++; break;
+	case '}': n--; break;
+	}
+    }
+    return n;
+}
+
 /*
  * Downcase a null-terminated string.
  */
